#include <iostream>
#include <ctime>
#include <utility>
#include <vector>
#include <list>
#include <string>

using namespace std;

//Zeichen     Hex    Okt
//========================
//   ''       8E     216
//   ''       84     204
//   ''       99     231
//   ''       94     224
//   ''       9A     232
//   ''       81     201
//   ''       E1     341

class date
{
private:
	short day_;
	short month_;
	unsigned int year_;
public:
	date() : day_(1), month_(1), year_(1900)
	{
	}

	date(short day, short month, unsigned int year) : day_(day), month_(month), year_(year)
	{
	}

	int get_age() const
	{
		time_t timestamp; //Aktuelles Datum und Uhrzeit in  Variable speichern
		timestamp = time(nullptr);
		tm date{};
		localtime_s(&date, &timestamp);
		const auto current_year = date.tm_year + 1900;
		return current_year - year_;
	}

	string get_birthday() const
	{
		return std::to_string(day_) + "." + to_string(month_) + "." + to_string(year_);
	}

	bool operator ==(const date& d) const { return day_ == d.day_ && month_ == d.month_ && year_ == d.year_; }
	bool operator !=(const date& d) const { return !operator==(d); }
};

/**
 * \brief Employee defined with name, first name & date of birth
 */
class employee
{
public:
	/**
	 * \brief Creates a new instance of employee
	 */
	employee(): holidays(0), taken_holidays(0)
	{
	}

	/**
	 * \brief Creates a new instance of employee
	 */
	employee(string name, string first_name, short day, short month, unsigned int year) : name(move(name)),
	                                                                                      first_name(move(first_name)),
	                                                                                      taken_holidays(0)
	{
		day_of_birth = date(day, month, year);
		const auto age = day_of_birth.get_age();
		holidays = age > 50 ? 32 : 30;
	}

	/**
	 * \brief Try take holidays
	 * \param days Days to take.
	 * \return True if holidays ready to take. False otherwise.
	 */
	bool try_take_holidays(const int days)
	{
		if (holidays - taken_holidays - days > 0)
		{
			taken_holidays += days;
			return true;
		}
		return false;
	}

	/**
	 * \brief Implements the equality operator.
	 */
	bool operator ==(const employee& e) const
	{
		return name == e.name && first_name == e.first_name && day_of_birth == e.day_of_birth;
	}

	/**
	 * \brief Implements the inequality operator.
	 */
	bool operator !=(const employee& e) const { return !operator==(e); }

	string name;
	string first_name;
	date day_of_birth;
	int holidays;
	int taken_holidays;
};

/**
 * \brief Collection of all employees (Database)
 */
static list<employee> employees;
/**
 * \brief Create new employee entry in database.
 */
void create_employee();
/**
 * \brief Search employee in the database
 * \return Return pointer of type employee
 */
employee* search_employee();
/**
 * \brief List all employees in database
 */
void list_employees();
/**
 * \brief Modify entry in database
 */
void modify_employee();
/**
 * \brief Print information about employee
 * \param employee Entry from database.
 */
void print_employee(employee employee);
/**
 * \brief Enter holidays to take, if available.
 * \param employee Employee as reference.
 */
void enter_holidays(employee& employee);

int main()
{
	auto end_program = false;
	while (!end_program)
	{
		system("cls");
		cout << "Urlaubsverwaltung (" << employees.size() << " Mitarbeiter)" << endl << endl;
		cout << "Bitte w\x84hlen Sie eine Option:" << endl;
		cout << "1: Mitarbeiter anlegen" << endl;
		cout << "2: Mitarbeiter bearbeiten" << endl;
		cout << "3: Alle Mitarbeiter auflisten" << endl;
		cout << "0: Programm beenden" << endl;
		cout << "ihre Eingabe:";
		auto input = 0;
		cin >> input;

		switch (input)
		{
		case 1:
			create_employee();
			break;
		case 2:
			modify_employee();
			break;
		case 3:
			list_employees();
			break;
		case 0:
			end_program = true;
			break;
		default:
			cout << "Unerlaubte Eingabe" << endl;
			system("pause");
		}
	}
	return 0;
}

void create_employee()
{
	while (true)
	{
		string name;
		cout << "Bitte einen Namen eingeben: ";
		cin >> name;
		string first_name;
		cout << "Bitte einen Vornamen eingeben: ";
		cin >> first_name;
		cout << "Bitte das Geburtsdatum eingeben: " << endl;
		short day;
		cout << "Tag: ";
		cin >> day;
		short month;
		cout << "Monat: ";
		cin >> month;
		unsigned int year;
		cout << "Jahr: ";
		cin >> year;
		employees.emplace_back(name, first_name, day, month, year);
		cout << "Einen weiteren Benutzer eingeben? [J/N]" << endl;
		string input;
		cin >> input;
		if (input._Equal("N") || input._Equal("n")) break;
	}
}


employee* search_employee()
{
	if (employees.empty()) throw 1;
	string name;
	cout << "Bitte einen Namen eingeben: ";
	cin >> name;
	string first_name;
	cout << "Bitte einen Vornamen eingeben: ";
	cin >> first_name;
	for (auto& e : employees)
	{
		if (e.first_name._Equal(first_name) && e.name._Equal(name))
		{
			return &e;
		}
	}
	throw 0;
}

void list_employees()
{
	for (auto& employee : employees)
	{
		print_employee(employee);
	}
	system("pause");
}

void modify_employee()
{
	auto end_program = false;
	try
	{
		const auto current = search_employee();

		while (!end_program)
		{
			system("cls");
			print_employee(*current);
			cout << "Bitte w\x84hlen Sie eine Option:" << endl;
			cout << "1: Urlaub eingeben" << endl;
			cout << "2: Mitarbeiter l\x94schen" << endl;
			cout << "0: Zur\x81 \bck" << endl;
			cout << "Ihre Eingabe:";
			auto input = 0;
			cin >> input;

			switch (input)
			{
			case 1:
				enter_holidays(*current);
				break;
			case 2:
				employees.remove(*current);
				cout << "Benutzer gel\x94scht";
				system("pause");
				end_program = true;
				break;
			case 0:
				end_program = true;
				break;
			default:
				cout << "Unerlaubte Eingabe" << endl;
				system("pause");
			}
		}
	}
	catch (int ex)
	{
		switch (ex)
		{
		case 0:
			cout << "Benutzer nicht gefunden. Code: " << ex << endl;
			break;
		case 1:
			cout << "Es gibt keine Benutzer in der Datenbank. Code: " << ex << endl;
			break;
		default:
			cout << "Unbekannter Fehler. Code: " << ex << endl;
		}
		system("pause");
	}
}

void print_employee(employee employee)
{
	cout << "    " << employee.first_name << " " << employee.name << endl;
	cout << "    Geburtstag: " << employee.day_of_birth.get_birthday() << endl;
	cout << "    Resturlaub: " << employee.holidays - employee.taken_holidays << endl;
	cout << "--------------------------------" << endl;
}

void enter_holidays(employee& employee)
{
	while (true)
	{
		int holidays;
		cout << "Urlaubstage eingeben: ";
		cin >> holidays;
		if (employee.try_take_holidays(holidays))
		{
			cout << "Urlaub genehmingt." << endl;
		}
		else
		{
			cout << "Urlaub abgelehnt." << endl;
		}
		cout << "Resturlaub: " << employee.holidays - employee.taken_holidays << " Tage" << endl;
		cout << "Einen weiteren Urlaubsantrag eingeben? [J/N]" << endl;
		string input;
		cin >> input;
		if (input._Equal("N") || input._Equal("n")) break;
	}
}
